use modules::EventLoop;
use toml::value::Table;

use std::sync::Arc;
use std::sync::mpsc::Receiver;
use transformable_channels::mpsc::ExtSender;

use helpers::command::split_command;
use event::{Event, Envelope};
use i_can_has_cat::Cats;

pub struct CatModule {
    prefix: String
}

impl CatModule {
    pub fn new (_: &Table, configuration: &Table) -> Box<EventLoop> {
        let prefix = configuration.get("prefix")
                                  .and_then(|value| value.as_str())
                                  .unwrap_or("?cat");

        Box::new(CatModule {
            prefix: String::from(prefix)
        })
    }
}

impl EventLoop for CatModule {
    fn run(&self, _: Box<ExtSender<Event>>, receiver: Receiver<Arc<Envelope>>) {
        let cats = Cats::new();

        loop {
            match receiver.recv() {
                Ok(envelope) => {
                    match envelope.event {
                        Event::Message { ref message } => {
                            debug!("Received message from module {:?}... {:?}", envelope.from, message.content);
                            match split_command(&message.content) {
                                Some((command, argument)) => {
                                    if command == self.prefix {
                                        let cat = if !argument.is_empty() {
                                            cats.random_image_of_category(argument)
                                        } else {
                                            cats.random_image()
                                        };

                                        if let Ok(cat) = cat {
                                            message.reply(&cat.url);
                                        }
                                    }
                                },
                                _ => {}
                            }
                        }
                        _ => ()
                    }
                }
                Err(error) => { error!("Error {:?}", error) }
            }
        }
    }
}
